@@ -1,32 +1,82 @@
 library(dplyr)
 
-test_that("custom renderers - ecr.ebars grouped", {
+test_that("ecr.ebars", {
+  # dset + groups
+  p <- iris |> group_by(Species) |>
+    ec.init(load= 'custom', ctype='bar', legend=list(show=T), yAxis=list(type='category')) |>
+    ecr.ebars(encode= list(x=c(2,3,4), y=1))
+  expect_equal(p$x$opts$series[[4]]$z, 3)
+  expect_match(p$x$opts$series[[4]]$tooltip$formatter, 'range', fixed=TRUE)
+  
+  # dset + groups
   df <- mtcars |> group_by(cyl,gear) |> summarise(yy= round(mean(mpg),2)) |>
-    mutate(low= round(yy-cyl*runif(1),2), high= round(yy+cyl*runif(1),2)) |>
-    relocate(cyl, .after = last_col())   # move group column behind first four cols
-  
-  p <- df |> ec.init(ctype='bar', load='custom', xAxis=list(type='category')) |>
-    ecr.ebars(df)
-  
+    mutate(low= round(yy-cyl*runif(1),2), high= round(yy+cyl*runif(1),2))
+  p <- ec.init(df, load='custom', ctype='bar', tooltip= list(s=T), xAxis=list(type='category')) |> 
+    ecr.ebars(encode= list(y=c(3,4,5), x=2))
+  expect_equal(p$x$opts$series[[6]]$name, '8')
+  expect_match(p$x$opts$series[[6]]$tooltip$formatter, 'range', fixed=TRUE)
   expect_equal(length(p$x$opts$series), 6)
   expect_equal(p$x$opts$series[[4]]$type, 'custom')
-  expect_true( p$x$opts$series[[4]]$renderItem == "sessionStorage.setItem('ErrorBar.oss','[\"\",\"\",\"3\",\"6\"]'); riErrorBar;")
+  expect_true(p$x$opts$series[[4]]$renderItem == 'riErrBars')
   expect_s3_class(p$x$opts$series[[4]]$renderItem, 'JS_EVAL')
   expect_equal(p$x$opts$xAxis$type, 'category')
-  expect_equal(as.character(p$x$opts$series[[6]][['name']]), '8')
+
+  # test for auto xAxis= list(type='category')
+  p <- Orange |> arrange(Tree) |> mutate(  #Tree= paste0('D',Tree), # char
+          up= circumference+runif(5)*6,
+          lo= circumference-runif(5)*6 ) |> group_by(age) |> 
+    ec.init(load='custom', ctype='bar', legend= list(show=T),
+      series.param= list(encode= list(x='Tree', y='circumference'))           
+    ) |> ecr.ebars(encode= list(x=1, y=c(3,4,5)))
+  expect_equal(p$x$opts$xAxis$type, 'category')
+
+  # data + name + char.encode
+  p <- ec.init(load= 'custom', legend= list(show=T), tooltip= list(show=T), 
+               xAxis=list(type='category'),
+    series= list(list(type='bar', name= 'data',
+      dimensions= c('cyl','gear','yy','low','high'), encode= list(x='gear',y='yy'),
+      data= ec.data(df |> filter(cyl==4))
+  ))) |> 
+    ecr.ebars(encode= list(x='gear', y=c('yy','low','high')), hwidth=12, name='err',
+      itemStyle= list(borderWidth= 2.5, color= "red")
+  )
+  expect_equal(p$x$opts$series[[2]]$encode$y, c(2,3,4))
+  expect_equal(p$x$opts$series[[2]]$itemStyle$borderDashOffset, 12)
+
+  # grouped + non-categorical
+  tmp <- round(rnorm(24, sin(1:24/2)*10, .5))
+  df <- data.frame(x = 1:24, val = tmp,
+         lower = round(rnorm(24, tmp -10, .5)),
+         upper = round(rnorm(24, tmp + 5, .8)),
+         cat= rep(c('A','B'),24) ) |> group_by(cat)
+  p <- ec.init(df, load='custom') |> ecr.ebars(encode= list(x=1, y=c(2,3,4)))
+  expect_equal(p$x$opts$series[[3]]$encode$y, c(1,2,3))
+    # make horizontal
+  p <- ec.init(df, load='custom', series.param= list(encode= list(y=1, x=2))) |> 
+    ecr.ebars(encode= list(y=1, x=c(2,3,4)))     #|> ec.inspect()
+  expect_equal(p$x$opts$series[[1]]$encode$y, 0)
+  expect_equal(p$x$opts$series[[3]]$encode$x, c(1,2,3))
+
+  # manual series + data
+  sers = list(
+    list(name= 's1',type= 'bar', data= list(5, 20, 36)),
+    list(name= 's2',type= 'bar', data= list(4, 2, 40)),
+    list(name= 's3',type= 'bar', data= list(15, 10, 36)),
+    list(name= 's2',type= 'custom', z=11, renderItem= htmlwidgets::JS('riErrBars'), 
+         itemStyle= list(color= 'brown', borderDashOffset=8 ), # 8= halfWidth
+         encode= list(x=1,y=c(2,3,4)),
+         data= list(list('Mon',5,4,7), list('Tue',10, 9, 11))
+    )
+  )
+  p <- ec.init(load='custom', yAxis= list(show=T),
+               xAxis= list(data= list('Mon','Tue','Wed')),
+               series= sers
+  )
+  expect_equal(p$x$opts$series[[4]]$encode$y, c(1,2,3))
+  expect_s3_class(p$x$opts$series[[4]]$renderItem, 'JS_EVAL')
 })
 
-test_that("custom renderers - ecr.ebars non-grouped", {
-  df <- iris |> distinct(Sepal.Length, .keep_all= TRUE) |> 
-    mutate(lo= Sepal.Width-Petal.Length/2, hi= Sepal.Width+Petal.Width) |>
-    select(Sepal.Length, Sepal.Width, lo, hi, Species)
-  p <- df |> ec.init(load='custom', legend=list(show=TRUE), xAxis=list(scale=TRUE)) |> 
-    ecr.ebars(name= 'err')
-  expect_equal(p$x$opts$series[[2]]$z, 3)
-  expect_match(p$x$opts$series[[2]]$tooltip$formatter, 'range', fixed=TRUE)
-})
-
-test_that("custom renderers - riErrBarSimple", {
+test_that("ecr.ebars riErrBarSimple", {
   set.seed(222)
   df <- data.frame(category= paste0('category', seq(1,50,1)),
                    avg= round(runif(50) * 1000, 2)) |>
@@ -55,21 +105,20 @@ test_that("custom renderers - riErrBarSimple", {
   expect_equal(length(p$x$opts$series[[2]]$data), 50)
 })
 
-test_that("custom renderers - ecr.band", {
+test_that("ecr.band", {
   df <- Orange |> mutate(Tree=as.numeric(Tree)) |> relocate(Tree, .after= last_col())
   band <- ecr.band(df |> filter(Tree==4) |> inner_join(df |> filter(Tree=='1'), by='age'),
         'circumference.y', 
         'circumference.x', 
         type= 'polygon', name= 'poly1')
-  p <- df |> group_by(Tree) |> ec.init(load='custom',
-    legend= list(s=T), tooltip= list(trigger='axis'), 
-    dataZoom= list(type='inside', filterMode='none')
+  p <- ec.init(load='custom',
+  	legend= list(s=T), tooltip= list(trigger='axis'), 
+  	dataZoom= list(type='inside', filterMode='none'),
+  	series= list( band[[1]],
+  		list(type='line', data=ec.data(df |> filter(Tree==5)), 
+  			color='orange', name='line1', clip=F)
+  	)
   )
-  p$x$opts$series <- list(
-    band[[1]],
-    list(type='line', datasetIndex=3, color='orange', name='line1', clip=F)
-  )
-  
   expect_equal(length(p$x$opts$series), 2)
   expect_equal(p$x$opts$series[[1]]$type, 'custom')
   expect_equal(p$x$opts$series[[1]]$name, 'poly1')
@@ -77,7 +126,7 @@ test_that("custom renderers - ecr.band", {
   expect_s3_class(p$x$opts$series[[1]]$renderItem, 'JS_EVAL')
 })
 
-test_that("custom renderers - ecr.band tooltips", {
+test_that("ecr.band tooltips", {
   df <- airquality |> mutate(lwr= round(Temp-Wind*2),
                              upr= round(Temp+Wind*2),
                              x= paste0(Month,'-',Day) ) |>
@@ -101,7 +150,7 @@ test_that("custom renderers - ecr.band tooltips", {
   expect_match(p$x$opts$tooltip$formatter, 'ss=[2.3,0.2,1.2]', fixed=TRUE)
 })
 
-test_that("custom renderers - geoJson", {
+test_that("leaflet with geoJson", {
   myGeojson= gsub('\n', '', '{
     "type": "FeatureCollection",
     "features": [
